#!/bin/sh

dir_this="/share/Apps/LTU"
dir_root="/usr/ltu"

func_stop()
{
	telnetd_running=0
	
	# close microdc2
	killall -q microdc2
	
	# close aria2c
	killall -q aria2c
	
	# close smartd
	killall -q smartd
	
	# close ssh
	killall -q dropbear
	
	# close mc
	killall -q mc
	
	# close logger
	killall -q klogd
	killall -q syslogd
	
	if [ -n "`pidof telnetd`" ] ; then
		killall -q telnetd
		telnetd_running=1
	fi
	
	# remove SSH dropbear
	rm /usr/bin/scp 2>/dev/null
	rm -f /usr/bin/dropbearkey
	rm -f /usr/bin/dropbearconvert
	rm -f /usr/bin/dropbearmulti
	rm -f /usr/sbin/dropbear
	rm -rf /etc/dropbear
	
	# restore busybox links
	rm -f /bin/busybox.ext
	rm -f /usr/bin/top
	rm -f /usr/sbin/telnetd
	ln -s /bin/busybox /usr/bin/top
	ln -s /bin/busybox /usr/sbin/telnetd
	
	# delete symlink
	rm -f "$dir_root"
	rm -f /usr/lib/locale
	
	# unbind and restore setups.cgi
	umount /opt/sybhttpd/default/setups.cgi 2>/dev/null
	cp -f /opt/syb/app/opt/sybhttpd/default/setups.cgi /opt/sybhttpd/default 2>/dev/null
	
	# restore telnetd
	if [ $telnetd_running -eq 1 ] ; then
		telnetd -l /bin/sh& 2>/dev/null
	fi
}

func_start()
{
	# set default params
	LOCALE_UTF8=1
	TERMINAL_SERVER=1
	PASSWD_ROOT="1234"
	START_LOGGER=0
	START_DCPP=0
	START_ARIA_RPC=0
	START_SMARTD=0
	START_NUT=0
	USE_FIX_TCP_RMEM=0
	HDD_SPINDOWN_TIME=30
	HDD_APM_DISABLE=0
	HDD_AAM_QUIET=0
	
	# close unsafe telnet from syabas init.d
	telnetd_running=0
	if [ -n "`pidof telnetd`" ] ; then
		killall -q telnetd
		telnetd_running=1
	fi
	
	# close stupid nmbd call from syabas /etc/init.d/S70nmt
	nmbd_pids=`pidof nmbd`
	for nmbd_pid in $nmbd_pids ; do
		if [ -n "$nmbd_pid" ] ; then
			nmbd_cmd=`cat /proc/$nmbd_pid/cmdline | grep '/nmt/apps/bin/nmbd'`
			[ -z "$nmbd_cmd" ] && kill $nmbd_pid
		fi
	done
	
	# create symlink for ltu
	ln -snf "$dir_this" "$dir_root"
	
	# create symlink for locale
	ln -snf "$dir_this/lib/locale" /usr/lib/locale
	
	# extend path
	export PATH="$dir_root/bin:$dir_root/sbin:$PATH"
	export LD_LIBRARY_PATH="$dir_root/lib:$LD_LIBRARY_PATH"
	
	# add profile extension
	if [ "`grep $dir_root/etc/profile /etc/profile`" = "" ] ; then
		echo ""  >> /etc/profile
		echo "[ -r $dir_root/etc/profile ] && . $dir_root/etc/profile" >> /etc/profile
	fi
	
	# copy additional busybox binary
	cp -f "$dir_this/updates/bin/busybox.ext" /bin
	chmod 775 /bin/busybox.ext
	
	# remove old busybox links
	rm /usr/bin/top
	rm -f /usr/bin/passwd
	rm -f /usr/sbin/telnetd
	
	# create additional busybox symlinks
	/bin/busybox.ext --install -s
	
	# copy hdparm wrapper
	cp -f "$dir_this/sbin/hdd-tune" /sbin
	cp -f "$dir_this/updates/sbin/hdparm" /sbin
	
	# load config
	if [ -f "$dir_this/configure" ] ; then
		dos2unix "$dir_this/configure"
		. "$dir_this/configure"
	fi
	
	# start system and kernel logger daemons
	if [ $START_LOGGER -ne 0 ] ; then
		file_logger="/tmp/messages"
		link_logger="$dir_this/log/messages.lnk"
		if [ $START_LOGGER -eq 2 ] ; then
			file_logger="$dir_this/log/messages.all"
			rm -f "$link_logger"
		fi
		touch "$file_logger"
		syslogd -O "$file_logger"
		klogd
		[ $START_LOGGER -eq 1 ] && [ ! -f "$link_logger" ] && ln -snf "$file_logger" "$link_logger"
	fi
	
	# link to internal wireless-tool
	ln -s "/opt/wifi/bin/iwconfig" /sbin/iwconfig 2>/dev/null
	ln -s "/opt/wifi/bin/iwpriv" /sbin/iwpriv 2>/dev/null
	
	# check pure-ftpd deferred update
	if [ ! -n "`pidof pure-ftpd`" ] ; then
		if [ -f /nmt/apps/bin/pure-pw.new ] ; then
			mv -f /nmt/apps/bin/pure-pw.new /nmt/apps/bin/pure-pw
		fi
		if [ -f /nmt/apps/bin/pure-ftpd.new ] ; then
			mv -f /nmt/apps/bin/pure-ftpd.new /nmt/apps/bin/pure-ftpd
			# generate new pureftpd.pdb
			/nmt/apps/bin/pure-pw mkdb /etc/pureftpd.pdb -f /etc/pureftpd.passwd
		fi
	fi
	
	if [ $TERMINAL_SERVER -ne 0 ] || [ $telnetd_running -eq 1 ] ; then
		# check PTY in kernel
		if [ "`cat /proc/filesystems | grep devpts`" = "" ] ; then
			kernel_ver=`uname -r`
			dir_src="$dir_this/lib/modules/$kernel_ver"
			dir_dst="/lib/modules/$kernel_ver/kernel/pty"
			mkdir -p "$dir_dst"
			for i in devpts-inode.ko modpty.ko ; do
				[ -f "$dir_src/$i" ] && cp -f "$dir_src/$i" "$dir_dst" && insmod "$dir_dst/$i"
			done
		fi
		
		# mount devpts
		if ! mountpoint -q /dev/pts ; then
			mount -t devpts devpts /dev/pts
		fi
		
		# allow securetty for ttyp0 ttyp1 ttyp2 ttyp3
		if [ "`grep 'ttyp0' /etc/securetty`" = "" ] ; then
			for i in 0 1 2 3 ; do
				echo "ttyp$i" >> /etc/securetty
			done
		fi
		
		# enable shell for user root
		if [ "`grep 'root:/bin/sh' /etc/passwd`" = "" ] ; then
			sed "s|root:/bin/true$|root:/bin/sh|" -i /etc/passwd
		fi
		
		# create system password for root
		if [ "$PASSWD_ROOT" != "" ] ; then
			echo "$PASSWD_ROOT" >  /tmp/.pwd_root
			echo "$PASSWD_ROOT" >> /tmp/.pwd_root
			cat /tmp/.pwd_root | passwd root >/dev/null
			rm -f /tmp/.pwd_root
		fi
		
		# start telnetd daemon
		if [ $TERMINAL_SERVER -eq 1 ] || [ $telnetd_running -eq 1 ] ; then
			/usr/sbin/telnetd -K -l login
		fi
		
		# start ssh server
		if [ $TERMINAL_SERVER -eq 2 ] ; then
			# create ssh keys if needed
			if [ ! -f "$dir_this/etc/dropbear/rsa_key" ] ; then
				echo "Create Dropbear RSA key..."
				"$dir_this/bin/dropbearkey" -t rsa -f "$dir_this/etc/dropbear/rsa_key" >/dev/null
			fi
			
			if [ ! -f "$dir_this/etc/dropbear/dss_key" ] ; then
				echo "Create Dropbear DSS key..."
				"$dir_this/bin/dropbearkey" -t dss -f "$dir_this/etc/dropbear/dss_key" >/dev/null
			fi
			
			# copy dropbear to rootfs
			mkdir -p /usr/libexec
			mkdir -p /etc/dropbear
			cp -f "$dir_this/etc/dropbear/rsa_key" /etc/dropbear/dropbear_rsa_host_key
			cp -f "$dir_this/etc/dropbear/dss_key" /etc/dropbear/dropbear_dss_host_key
			chmod 600 /etc/dropbear/dropbear_rsa_host_key
			chmod 600 /etc/dropbear/dropbear_dss_host_key
			
			cp -f "$dir_this/bin/dropbearmulti" /usr/bin
			cp -f "$dir_this/lib/openssh/sftp-server" /usr/libexec
			ln -sf /usr/bin/dropbearmulti /usr/bin/scp
			ln -sf /usr/bin/dropbearmulti /usr/bin/dropbearkey
			ln -sf /usr/bin/dropbearmulti /usr/bin/dropbearconvert
			ln -sf /usr/bin/dropbearmulti /usr/sbin/dropbear
			
			touch /tmp/lastlog
			/usr/sbin/dropbear -W 65536
		fi
	fi
	
	# patch setups.cgi (disable background changes tcp_rmem)
	umount /opt/sybhttpd/default/setups.cgi 2>/dev/null
	"$dir_this/bin/patch_cgi" /opt/sybhttpd/default/setups.cgi
	
	# increase read socket buffer size to 0.5 MB
	rmem_max=`cat /proc/sys/net/core/rmem_max`
	rmem_def=`cat /proc/sys/net/core/rmem_default`
	[ $rmem_max -lt 524288 ] && echo 524288 >/proc/sys/net/core/rmem_max
	[ $rmem_def -lt 524288 ] && echo 524288 >/proc/sys/net/core/rmem_default
	
	# fix TCP receive window (prevent kernel crash)
	if [ $USE_FIX_TCP_RMEM -eq 1 ] ; then
		echo "16384 87380 87380" >/proc/sys/net/ipv4/tcp_rmem
	fi
	
	# start DC++ microdc2
	if [ $START_DCPP -eq 1 ] ; then
		microdc2d
	fi
	
	# start aria2c as RPC server
	if [ $START_ARIA_RPC -eq 1 ] ; then
		aria2d
	fi
	
	# start UPS NUT
	if [ $START_NUT -eq 1 ] ; then
		nut-start
	fi
	
	# tune HDD
	/sbin/hdd-tune
	
	# start smartd daemom, check SMART every 2 hour
	if [ $START_SMARTD -eq 1 ] && [ $START_LOGGER -ne 0 ] ; then
		smartd -i 7200
	fi
	
	# call user script
	if [ -x "$dir_this/user.scripts/on_start_nmt.sh" ] ; then
		"$dir_this/user.scripts/on_start_nmt.sh"
	fi
}

case "$1" in
start)
	func_start
	;;
stop)
	func_stop
	;;
restart)
	func_stop
	sleep 1
	func_start
	;;
esac
