#!/bin/sh

dir_this="/share/Apps/LTU"
dir_apps="$dir_this/updates/nmt/apps"
bak_path="/nmt/apps/backup.ltu"
ver_trmd="2.42"
bld_trmd=13013

func_transmission_stop()
{
	# disable transmission
	if [ "`pflash get btpdserv`" = "1" ] ; then
		pflash set btpdserv 0
	fi
	
	# check transmission is started
	if [ -n "`pidof transmission-daemon`" ] ; then
		# stop transmission
		/nmt/apps/server/clutch stop
	fi
	
	killall -q btpd_monitor
	
	# gracefully wait max 10 seconds while transmission stopped (for 1.76)
	i=0
	while [ -n "`pidof transmission-daemon`" ] && [ $i -le 10 ] ; do
		i=$(( $i + 1 ))
		sleep 1
	done
}

func_transmission_update()
{
	# check NMT installed
	[ ! -d /nmt/apps ] && return 1
	
	# check transmission-deamon version
	bld_daemon=`/nmt/apps/bin/transmission-daemon -V 2>&1 | grep 'transmission-daemon ' |  cut -d'(' -f2 | cut -d')' -f1`
	if [ "$bld_daemon" != "" ] && [ $bld_daemon -ge $bld_trmd ] && [ -f "$bak_path/bin/transmission-remote" ] ; then
		# update only transmission caller
		cp -f "$dir_apps/server/clutch" /nmt/apps/server
		cp -f "$dir_apps/server/clutch" /nmt/apps/server/clutch_backup
		return 0
	fi
	
	echo "Update NMT transmission-daemon to new version ($ver_trmd)..."
	
	func_transmission_stop
	
	# backup native NMT transmission
	if [ ! -f "$bak_path/bin/transmission-daemon" ] ; then
		mkdir -p "$bak_path/bin"
		mkdir -p "$bak_path/lib"
		mkdir -p "$bak_path/server"
		if [ -f /nmt/apps/server/clutch_backup ] ; then
			# use backup from CSI
			cp -f /nmt/apps/server/clutch_backup "$bak_path/server/clutch"
		else
			cp -f /nmt/apps/server/clutch "$bak_path/server"
		fi
		mv -f /nmt/apps/lib/libcurl.so.4 "$bak_path/lib"
		mv -f /nmt/apps/bin/transmission-daemon "$bak_path/bin"
		mv -f /nmt/apps/bin/transmission-remote "$bak_path/bin"
		mv -f /nmt/apps/clutch5 "$bak_path"
	fi
	
	[ ! -f "$bak_path/bin/transmission-remote" ] && mv -f /nmt/apps/bin/transmission-remote "$bak_path/bin"
	
	# delete native NMT transmission caller
	rm -f /nmt/apps/server/clutch
	
	# delete native NMT transmission binaries
	rm -f /nmt/apps/bin/transmission-daemon
	rm -f /nmt/apps/bin/transmission-remote
	
	# delete native NMT libraries
	rm -f /nmt/apps/lib/libcurl.so
	rm -f /nmt/apps/lib/libcurl.so.4
	
	# delete native NMT transmission web interface
	if [ -d /nmt/apps/clutch5 ] ; then
		rm -rf /nmt/apps/clutch5
	fi
	
	# copy new library
	cp -f "$dir_apps/lib/libevent-2.0.so.5" /nmt/apps/lib
	cp -f "$dir_apps/lib/libcurl.so.4" /nmt/apps/lib
	ln -f /nmt/apps/lib/libcurl.so.4 /nmt/apps/lib/libcurl.so
	
	# create library symlinks
	ln -sf /nmt/apps/lib/libcurl.so /lib/libcurl.so
	ln -sf /nmt/apps/lib/libcurl.so.4 /lib/libcurl.so.4
	ln -sf /nmt/apps/lib/libevent-2.0.so.5 /lib/libevent-2.0.so.5
	
	# copy new transmission binaries
	cp -f "$dir_apps/bin/transmission-daemon" /nmt/apps/bin
	cp -f "$dir_apps/bin/transmission-remote" /nmt/apps/bin
	
	# copy new transmission web interface
	cp -rf "$dir_apps/clutch5" /nmt/apps
	
	# copy new NMT transmission caller
	cp -f "$dir_apps/server/clutch" /nmt/apps/server
	cp -f "$dir_apps/server/clutch" /nmt/apps/server/clutch_backup
	
	# try to copy torrents and settings from CSI Transmisson
	nmt_conf="/share/.transmission"
	mkdir -p -m 777 "$nmt_conf"
	if [ ! -f "$nmt_conf/.copied_from_csi" ] ; then
		csi_conf="/share/Apps/Transmission/.transmission"
		for i in "$csi_conf/settings.json" "$csi_conf/stats.json" "$csi_conf/dht.dat" ; do
			if  [ -f "$i" ] ; then
				cp -f "$i" "$nmt_conf"
				[ $? -eq 0 ] && echo "$i" >> "$nmt_conf/.copied_from_csi"
			fi
		done
		for i in "$csi_conf/resume" "$csi_conf/torrents" ; do
			if  [ -d "$i" ] ; then
				cp -r "$i" "$nmt_conf"
				[ $? -eq 0 ] && echo "$i" >> "$nmt_conf/.copied_from_csi"
			fi
		done
	fi
	
	# create default settings.json
	if [ ! -f "$nmt_conf/settings.json" ] ; then
		/nmt/apps/bin/transmission-daemon -a "127.0.0.1, *.*.*.*" -w /share/Download -c /share/Watch -p 9091 -ep -L 100 -l 20 --no-utp -d 2>/tmp/settings.json
		mv /tmp/settings.json "$nmt_conf/settings.json"
	fi
	
	if [ -f "$nmt_conf/settings.json" ] ; then
		chmod 666 "$nmt_conf/settings.json"
	fi
	
	# try to disable CSI Transmisson
	if [ -d /share/Apps/Transmission ] ; then
		/share/Apps/AppInit/appinit.cgi disable Transmission
	fi
	
	echo "Done."
}

func_transmission_restore()
{
	# check NMT installed
	[ ! -d /nmt/apps ] && return 1
	
	# restore transmission-daemon to native NMT version
	
	func_transmission_stop
	
	# restore from backup
	if [ -f "$bak_path/lib/libcurl.so.4" ] ; then
		rm -f /nmt/apps/lib/libcurl.so
		cp -f "$bak_path/lib/libcurl.so.4" /nmt/apps/lib
		ln -f /nmt/apps/lib/libcurl.so.4 /nmt/apps/lib/libcurl.so
	fi
	
	[ -f "$bak_path/bin/transmission-daemon" ] && cp -f "$bak_path/bin/transmission-daemon" /nmt/apps/bin
	[ -f "$bak_path/bin/transmission-remote" ] && cp -f "$bak_path/bin/transmission-remote" /nmt/apps/bin
	
	if [ -d "$bak_path/clutch5" ] ; then
		rm -rf /nmt/apps/clutch5
		cp -rf "$bak_path/clutch5" /nmt/apps
	fi
	
	if [ -f "$bak_path/server/clutch" ] ; then
		cp -f "$bak_path/server/clutch" /nmt/apps/server
		cp -f "$bak_path/server/clutch" /nmt/apps/server/clutch_backup
	fi
}

func_pureftpd_update()
{
	# check NMT installed
	[ ! -d /nmt/apps ] && return 1
	
	# check old pure-ftpd version
	invalid_ver=`/nmt/apps/bin/pure-ftpd -h 2>&1 | grep '1.0.21'`
	if [ -z "$invalid_ver" ] ; then
		# update only pure-ftpd caller
		cp -f "$dir_apps/etc/ftpserver.sh" /nmt/apps/etc
		return 0
	fi
	
	echo "Update NMT pure-ftpd to new version..."
	
	# backup native NMT pure-ftpd
	if [ ! -f "$bak_path/bin/pure-ftpd" ] ; then
		mkdir -p "$bak_path/bin"
		mkdir -p "$bak_path/etc"
		cp -f /nmt/apps/bin/pure-ftpd "$bak_path/bin"
		cp -f /nmt/apps/bin/pure-pw "$bak_path/bin"
		cp -f /nmt/apps/etc/ftpserver.sh "$bak_path/etc"
	fi
	
	rm -f /nmt/apps/etc/ftpserver.sh
	cp -f "$dir_apps/etc/ftpserver.sh" /nmt/apps/etc
	
	if [ ! -n "`pidof pure-ftpd`" ] ; then
		# delete native NMT pure-ftpd
		rm -f /nmt/apps/bin/pure-ftpd
		rm -f /nmt/apps/bin/pure-pw
		
		# copy new pure-ftpd
		cp -f "$dir_apps/bin/pure-ftpd" /nmt/apps/bin
		cp -f "$dir_apps/bin/pure-pw" /nmt/apps/bin
		
		rm -f /nmt/apps/bin/pure-ftpd.new
		rm -f /nmt/apps/bin/pure-pw.new
		
		# generate new pureftpd.pdb
		/nmt/apps/bin/pure-pw mkdb /etc/pureftpd.pdb -f /etc/pureftpd.passwd
		
		echo "Done."
	else
		cp -f "$dir_apps/bin/pure-ftpd" /nmt/apps/bin/pure-ftpd.new
		cp -f "$dir_apps/bin/pure-pw" /nmt/apps/bin/pure-pw.new
		
		echo "Done, pure-ftpd will be complete updated after next NMT boot."
	fi
}

func_pureftpd_restore()
{
	# check NMT installed
	[ ! -d /nmt/apps ] && return 1
	
	# restore pure-ftpd to native NMT version
	
	# stop pure-ftpd
	/nmt/apps/etc/ftpserver.sh stop
	
	rm -f /nmt/apps/bin/pure-ftpd.new
	rm -f /nmt/apps/bin/pure-pw.new
	
	# restore from backup
	if [ -f "$bak_path/etc/ftpserver.sh" ] ; then
		cp -f "$bak_path/etc/ftpserver.sh" /nmt/apps/etc
	fi
	
	if [ -f "$bak_path/bin/pure-ftpd" ] ; then
		cp -f "$bak_path/bin/pure-ftpd" /nmt/apps/bin
	fi
	
	if [ -f "$bak_path/bin/pure-pw" ] ; then
		cp -f "$bak_path/bin/pure-pw" /nmt/apps/bin
	fi
	
	# generate new pureftpd.pdb
	/nmt/apps/bin/pure-pw mkdb /etc/pureftpd.pdb -f /etc/pureftpd.passwd
	
	if [ "`pflash get ftpserv`" = "1" ] ; then
		/nmt/apps/etc/ftpserver.sh start
	fi
}

func_tune_apache()
{
	for i in "/nmt/apps/server/php5server/httpd.conf" "/nmt/apps/server/httpd.conf" ; do
		if [ -f "$i" ] ; then
			sed "s/\(MinSpareServers *\).*/\11/" -i "$i"
			sed "s/\(MaxSpareServers *\).*/\11/" -i "$i"
			sed "s/\(StartServers *\).*/\11/" -i "$i"
			sed "s/\(MaxClients *\).*/\15/" -i "$i"
			echo "Apache conf file $i tuned!"
		fi
	done
}

func_backup_xenv()
{
	nmt_ver=`cat /proc/syabas/fw_version`
	[ -z "$nmt_ver" ] && nmt_ver="unknown_fw"
	dir_dst="/share/.backup_nand/$nmt_ver"
	
	if [ ! -f "$dir_dst/sigmblocka.bin" ] ; then
		echo "Backup XENV to [$dir_dst]:"
		mkdir -p "$dir_dst"
		dd if=/dev/sigmblocka of="$dir_dst/sigmblocka.bin" bs=131072 count=4
		echo "Done!"
	fi
}

func_install()
{
	LOCALE_UTF8=1
	TERMINAL_SERVER=1
	PASSWD_ROOT="1234"
	START_LOGGER=0
	START_DCPP=0
	START_ARIA_RPC=0
	START_SMARTD=0
	START_NUT=0
	USE_FIX_TCP_RMEM=0
	HDD_SPINDOWN_TIME=30
	HDD_APM_DISABLE=0
	HDD_AAM_QUIET=0
	
	# unbind setups.cgi (old usage)
	umount /opt/sybhttpd/default/setups.cgi 2>/dev/null
	
	mkdir -p "$dir_this/log"
	mkdir -p "$dir_this/home/.aria2"
	mkdir -p "$dir_this/etc/dropbear"
	mkdir -p "$dir_this/etc/ssh"
	mkdir -p "$dir_this/etc/nut"
	mkdir -p "$dir_this/etc/microdc2"
	mkdir -p "$dir_this/user.scripts"
	
	dir_ltuo="/share/Apps/LinuxUtils"
	if [ -d "$dir_ltuo" ] ; then
		# try to stop old location
		/share/Apps/AppInit/appinit.cgi stop LinuxUtils
		
		# load old configure
		[ -f "$dir_ltuo/configure" ] && . "$dir_ltuo/configure"
		
		# copy ssh keys
		[ -f "$dir_ltuo/ssh/etc/dropbear/rsa_key" ] && mv "$dir_ltuo/ssh/etc/dropbear/rsa_key" "$dir_this/etc/dropbear" 2>/dev/null
		[ -f "$dir_ltuo/ssh/etc/dropbear/dss_key" ] && mv "$dir_ltuo/ssh/etc/dropbear/dss_key" "$dir_this/etc/dropbear" 2>/dev/null
		
		# copy user script
		[ -f "$dir_ltuo/user.script.sh" ] && mv "$dir_ltuo/user.script.sh" "$dir_this/user.scripts/on_start_nmt.sh" 2>/dev/null
		
		[ -f "$dir_ltuo/transmission/on_stop.sh" ] && mv "$dir_ltuo/transmission/on_stop.sh" "$dir_this/user.scripts/on_stop_transmission.sh" 2>/dev/null
		
		# copy full system log
		[ -f "$dir_ltuo/log/messages.all" ] && mv "$dir_ltuo/log/messages.all" "$dir_this/log" 2>/dev/null
		
		# delete old location
		rm -rf "$dir_ltuo"
	else
		# load prev configure
		[ -f "$dir_this/configure" ] && . "$dir_this/configure"
	fi
	
	# delete old files
	rm -f "$dir_this/lib/libncurses.so"
	rm -f "$dir_this/lib/libncurses.so.5"
	rm -f "$dir_this/lib/libncurses.so.5.7"
	rm -f "$dir_this/lib/libparted.so.0"
	rm -f "$dir_this/lib/libparted.so.0.0.1"
	rm -f "$dir_this/etc/mc/Syntax"
	rm -f "$dir_this/etc/mc/mc.charsets"
	rm -f "$dir_this/etc/mc/mc.lib"
	rm -f "$dir_this/share/mc/mc.hint"
	rm -f "$dir_this/share/mc/mc.hint.ru"
	rm -f "$dir_this/share/mc/mc.hint.uk"
	rm -f "$dir_this/share/mc/mc.hlp"
	rm -f "$dir_this/share/mc/mc.hlp.ru"
	
	# update new home path
	if [ -d "$dir_this/var/.microdc2" ] ; then
		mv -f "$dir_this/var/.microdc2" "$dir_this/home"
	fi
	
	if [ -f "$dir_this/etc/microdc2/config" ] ; then
		mv -f "$dir_this/etc/microdc2/config" "$dir_this/etc/microdc2.conf"
		rmdir "$dir_this/etc/microdc2" 2>/dev/null
	fi
	
	now_path=`pwd`
	
	# create default conf files and user scripts
	cd "$dir_this/updates/etc"
	for i in * ; do
		[ -f "$i" ] && [ ! -f "$dir_this/etc/$i" ] && cp "$i" "$dir_this/etc"
	done
	
	cd "$dir_this/updates/etc/nut"
	for i in * ; do
		[ -f "$i" ] && [ ! -f "$dir_this/etc/nut/$i" ] && cp "$i" "$dir_this/etc/nut"
	done
	
	cd "$dir_this/updates/scripts"
	for i in *.sh ; do
		[ -f "$i" ] && [ ! -f "$dir_this/user.scripts/$i" ] && cp "$i" "$dir_this/user.scripts"
	done
	
	cd "$now_path"
	
	# generate new configure
	echo "
# select system locale (0=en_US.UTF-8, 1=ru_RU.UTF-8, 2=uk_UA.UTF-8)
LOCALE_UTF8=${LOCALE_UTF8}

# set terminal server (0=off, 1=Telnet, 2=SSH)
TERMINAL_SERVER=${TERMINAL_SERVER}

# set root password for telnet and ssh access (required for ssh)
PASSWD_ROOT=\"${PASSWD_ROOT}\"

# start system logger daemon (0=off, 1=log to tmpfs /tmp/messages, 2=log to drive /share/Apps/LTU/log/messages.all)
START_LOGGER=${START_LOGGER}

# start DC++ client microdc2 (as daemon via screen). Configure in /share/Apps/LTU/etc/microdc2.conf
START_DCPP=${START_DCPP}

# start aria2c RPC server. Configure in /share/Apps/LTU/etc/aria2.conf
START_ARIA_RPC=${START_ARIA_RPC}

# start Network UPS Tools (NUT)
START_NUT=${START_NUT}

# start smartd logger daemon (logger needed)
START_SMARTD=${START_SMARTD}

# fixed TCP receive window for prevent kernel crash (0=auto fix on start transmission/aria2/microdc2, 1=force fix on start LTU)
USE_FIX_TCP_RMEM=${USE_FIX_TCP_RMEM}

# set HDD idle spindown time in minutes for all HDD drives (0=disable spindown timer, valid values 5/10/15/20/30/60/90/120/150/180 min)
HDD_SPINDOWN_TIME=$HDD_SPINDOWN_TIME

# disable HDD advanced power management for decrease Load_Cycle_Count (0=not change, 1=disable APM)
HDD_APM_DISABLE=$HDD_APM_DISABLE

# set HDD advanced acoustic management to quiet mode (0=not change, 1=AAM quiet)
HDD_AAM_QUIET=$HDD_AAM_QUIET" > "$dir_this/configure"
	
	chmod 666 "$dir_this/configure"
}


case "$1" in
install)
	func_install
	func_transmission_update
	func_pureftpd_update
	func_backup_xenv
	func_tune_apache
	;;
nmt-update)
	func_transmission_update
	func_pureftpd_update
	func_tune_apache
	;;
nmt-restore)
	func_transmission_restore
	func_pureftpd_restore
	;;
esac
